{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2016-2020 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{% extends "invenio_communities/base.html" %}

{%- block javascript %}
  {{ super() }}
  {{ webpack['invenio-communities-records-curate.js'] }}
{%- endblock javascript %}

{% set active_header_menu_item = 'curate' %}

{%- block page_body %}
{% include "invenio_communities/header.html" %}
<div class="ui container">
  {% set community_id = community.id | string %}
  <div data-invenio-search-config='{{
    search_app_helpers.invenio_records_rest.generate(
      dict(
        endpoint_id="recid",
        app_id="communities-records-curate",
        hidden_params=[("community_pending", community_id)]
      )
    ) | tojson(indent=2) }}'></div>
</div>

{%- endblock page_body %}
