# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Integration with the Invenio-Requests module."""

from .resolver import CommunityResolver

__all__ = (
    'CommunityResolver',
)
